@@ -15,14 +15,13 @@ package storage
 
 import (
 	"encoding/gob"
-	"io"
+	"fmt"
 	"io/ioutil"
 	"os"
 	"path"
 	"sync"
 	"time"
 
-	"github.com/golang/protobuf/proto"
 	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
 
@@ -61,6 +60,8 @@ func NewDiskMetricStore(
 	persistenceFile string,
 	persistenceInterval time.Duration,
 ) *DiskMetricStore {
+	// TODO: Do that outside of the constructor to allow the HTTP server to
+	//  serve /-/healthy and /-/ready earlier.
 	dms := &DiskMetricStore{
 		writeQueue:      make(chan WriteRequest, writeQueueCapacity),
 		drain:           make(chan struct{}),
@@ -70,10 +71,6 @@ func NewDiskMetricStore(
 	}
 	if err := dms.restore(); err != nil {
 		log.Errorln("Could not load persisted metrics:", err)
-		log.Info("Retrying assuming legacy format for persisted metrics...")
-		if err := dms.legacyRestore(); err != nil {
-			log.Errorln("Could not load persisted metrics in legacy format: ", err)
-		}
 	}
 
 	go dms.loop(persistenceInterval)
@@ -95,7 +92,7 @@ func (dms *DiskMetricStore) GetMetricFamilies() []*dto.MetricFamily {
 
 	for _, group := range dms.metricGroups {
 		for name, tmf := range group.Metrics {
-			mf := tmf.MetricFamily
+			mf := tmf.GetMetricFamily()
 			stat, exists := mfStatByName[name]
 			if exists {
 				existingMF := result[stat.pos]
@@ -108,7 +105,7 @@ func (dms *DiskMetricStore) GetMetricFamilies() []*dto.MetricFamily {
 					result[stat.pos] = existingMF
 				}
 				if mf.GetHelp() != existingMF.GetHelp() || mf.GetType() != existingMF.GetType() {
-					log.Warnf(
+					log.Infof(
 						"Metric families '%s' and '%s' are inconsistent, help and type of the latter will have priority. This is bad. Fix your pushed metrics!",
 						mf, existingMF,
 					)
@@ -134,6 +131,26 @@ func (dms *DiskMetricStore) Shutdown() error {
 	return <-dms.done
 }
 
+// Healthy implements the MetricStore interface.
+func (dms *DiskMetricStore) Healthy() error {
+	// By taking the lock we check that there is no deadlock.
+	dms.lock.Lock()
+	defer dms.lock.Unlock()
+
+	// A pushgateway that cannot be written to should not be
+	// considered as healthy.
+	if len(dms.writeQueue) == cap(dms.writeQueue) {
+		return fmt.Errorf("write queue is full")
+	}
+
+	return nil
+}
+
+// Ready implements the MetricStore interface.
+func (dms *DiskMetricStore) Ready() error {
+	return dms.Healthy()
+}
+
 func (dms *DiskMetricStore) loop(persistenceInterval time.Duration) {
 	lastPersist := time.Now()
 	persistScheduled := false
@@ -142,7 +159,7 @@ func (dms *DiskMetricStore) loop(persistenceInterval time.Duration) {
 	var persistTimer *time.Timer
 
 	checkPersist := func() {
-		if !persistScheduled && lastWrite.After(lastPersist) {
+		if dms.persistenceFile != "" && !persistScheduled && lastWrite.After(lastPersist) {
 			persistTimer = time.AfterFunc(
 				persistenceInterval-lastWrite.Sub(lastPersist),
 				func() {
@@ -212,8 +229,8 @@ func (dms *DiskMetricStore) processWriteRequest(wr WriteRequest) {
 			dms.metricGroups[key] = group
 		}
 		group.Metrics[name] = TimestampedMetricFamily{
-			Timestamp:    wr.Timestamp,
-			MetricFamily: mf,
+			Timestamp:            wr.Timestamp,
+			GobbableMetricFamily: (*GobbableMetricFamily)(mf),
 		}
 	}
 }
@@ -234,6 +251,8 @@ func (dms *DiskMetricStore) GetMetricFamiliesMap() GroupingKeyToMetricGroup {
 }
 
 func (dms *DiskMetricStore) persist() error {
+	// Check (again) if persistence is configured because some code paths
+	// will call this method even if it is not.
 	if dms.persistenceFile == "" {
 		return nil
 	}
@@ -275,79 +294,24 @@ func (dms *DiskMetricStore) restore() error {
 	}
 	defer f.Close()
 	d := gob.NewDecoder(f)
-	return d.Decode(&dms.metricGroups)
-}
-
-func (dms *DiskMetricStore) legacyRestore() error {
-	if dms.persistenceFile == "" {
-		return nil
-	}
-	f, err := os.Open(dms.persistenceFile)
-	if os.IsNotExist(err) {
-		return nil
-	}
-	if err != nil {
+	if err := d.Decode(&dms.metricGroups); err != nil {
 		return err
 	}
-	defer f.Close()
 
-	var tmf TimestampedMetricFamily
-	for d := gob.NewDecoder(f); err == nil; tmf, err = legacyReadTimestampedMetricFamily(d) {
-		if len(tmf.MetricFamily.GetMetric()) == 0 {
-			continue // No metric in this MetricFamily.
-		}
-		name := tmf.MetricFamily.GetName()
-		var job, instance string
-		for _, lp := range tmf.MetricFamily.GetMetric()[0].GetLabel() {
-			// With the way the pushgateway persists things, all
-			// metrics in a single MetricFamily proto message share
-			// the same job and instance label. So we only have to
-			// peek at the first metric to find it.
-			switch lp.GetName() {
-			case "job":
-				job = lp.GetValue()
-			case "instance":
-				instance = lp.GetValue()
-			}
-			if job != "" && instance != "" {
-				break
+	// If we have decoded nothing into GobbableMetricFamily, we have
+	// probably hit the legacy disk format, where a MetricFamily was encoded
+	// (with subtle bugs) into the MetricFamily field. Move it over in that
+	// case. Next time we persist the data, it will be in the new format.
+	for _, mg := range dms.metricGroups {
+		for mn, tmf := range mg.Metrics {
+			if tmf.GobbableMetricFamily == nil {
+				tmf.GobbableMetricFamily = (*GobbableMetricFamily)(tmf.MetricFamily)
+				tmf.MetricFamily = nil
+				mg.Metrics[mn] = tmf
 			}
 		}
-		labels := map[string]string{
-			"job":      job,
-			"instance": instance,
-		}
-		key := model.LabelsToSignature(labels)
-		group, ok := dms.metricGroups[key]
-		if !ok {
-			group = MetricGroup{
-				Labels:  labels,
-				Metrics: NameToTimestampedMetricFamilyMap{},
-			}
-			dms.metricGroups[key] = group
-		}
-		group.Metrics[name] = tmf
 	}
-	if err == io.EOF {
-		return nil
-	}
-	return err
-}
-
-func legacyReadTimestampedMetricFamily(d *gob.Decoder) (TimestampedMetricFamily, error) {
-	var buffer []byte
-	if err := d.Decode(&buffer); err != nil {
-		return TimestampedMetricFamily{}, err
-	}
-	mf := &dto.MetricFamily{}
-	if err := proto.Unmarshal(buffer, mf); err != nil {
-		return TimestampedMetricFamily{}, err
-	}
-	var timestamp time.Time
-	if err := d.Decode(&timestamp); err != nil {
-		return TimestampedMetricFamily{}, err
-	}
-	return TimestampedMetricFamily{MetricFamily: mf, Timestamp: timestamp}, nil
+	return nil
 }
 
 func copyMetricFamily(mf *dto.MetricFamily) *dto.MetricFamily {
